require 'logstash-event'

module LogstashScribbler
  class Message
    attr_reader :error, :message, :custom_fields, :object
    def initialize(options)
      @error = inspected_error(options[:error])
      @message = options[:message]
      @custom_fields = options[:custom_fields]
      @object = options[:object]
    end

    def log
      LogStash::Event.new(data).to_json
    end

    def data
      log_msg = {
        :error => error,
        :message => message,
      }
      log_msg = custom_fields.merge!(log_msg) if custom_fields
      log_msg = namespaced_object.merge!(log_msg) if object
      log_msg.reject { |k, v| v.nil? }
    end

    def namespaced_object
      return { :object => object } unless is_active_record?(object)
      object_type = object.class.name
      {object_type.to_sym => object}
    end

    def inspected_error(error)
      if error.is_a? Exception
        e = error
        error = {:message => e.message, :inspect => e.inspect, :backtrace => e.backtrace}
      else
        error
      end
    end

    def is_active_record?(object)
      defined?(ActiveRecord::Base) && object.is_a?(ActiveRecord::Base)
    end
  end
end
